#ifndef NULLMAILER__FDBUF__H__
#define NULLMAILER__FDBUF__H__

#include "mystring.h"
#include "config.h"
#include <string.h>
#include <fcntl.h>

class fdbuf 
{
public:
  enum flagbits { flag_eof=1, flag_error=2, flag_closed=4 };

  fdbuf(int fdesc, bool dc, unsigned bufsize = BUFSIZE);
  ~fdbuf();
  bool error() const;
  bool closed() const;
  bool close();
protected:
  char* const buf;
  unsigned length;
  unsigned start;
  int errnum;
  unsigned flags;
  const unsigned size;
  const int fd;
  const bool do_close;
};

class fdobuf;

class fdibuf : protected fdbuf
{
public:
  fdibuf(const char* filename, unsigned bufsize = BUFSIZE);
  fdibuf(int fdesc, bool dc = false, unsigned bufsize = BUFSIZE);
  ~fdibuf();
  bool close() { return fdbuf::close(); }
  bool eof() const;
  bool operator!() const ;
  operator bool() const
    {
      return !operator!();
    }
  bool getchar(char& ch);
  bool getline(mystring& out, char terminator = '\n');
  bool getnetstring(mystring& out);
  bool read(char*, unsigned);
  unsigned last_count() { return count; }
  bool rewind();
protected:
  unsigned count;
  bool refill();
};

class fdobuf : protected fdbuf
{
public:
  enum openflags { create=O_CREAT, 
		   excl=O_EXCL,
		   trunc=O_TRUNC,
		   append=O_APPEND,
		   sync=O_SYNC,
		   nonblock=O_NONBLOCK };

  fdobuf(const char* filename, openflags,
	 unsigned bufsize = BUFSIZE);
  fdobuf(int fdesc, bool dc=false, unsigned bufsize = BUFSIZE);
  ~fdobuf();
  bool close() { return flush() && fdbuf::close(); }
  bool operator!() const;
  operator bool() const
    {
      return !operator!();
    }
  bool flush();
  bool write(const char*, unsigned);
  bool write(const char* str)
    {
      return write(str, strlen(str));
    }
  bool write(const mystring& str)
    {
      return write(str.c_str(), str.length());
    }
  bool writeln();
  bool writeln(const char* str);
  bool writeln(const mystring& str);
  unsigned last_count() { return count; }
protected:
  unsigned count;
};

bool fdbuf_copy(fdibuf&, fdobuf&, bool noflush = false);

extern fdobuf fout;
extern fdobuf ferr;
extern fdibuf fin;

#endif // NULLMAILER__FDBUF__H__
