#include <getopt.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "fdbuf.h"
#include "defines.h"

#ifndef HAVE_SETENV
// This is not really a full emulation of setenv, but close enough
int setenv(const char* var, const char* val, int overwrite)
{
  size_t varlen = strlen(var);
  size_t vallen = strlen(val);
  char str[varlen+vallen+2];
  memcpy(str, var, varlen);
  str[varlen] = '=';
  memcpy(str+varlen+1, val, vallen);
  str[varlen+vallen+1] = 0;
  return putenv(str);
}
#endif

bool setenvelope(char* str)
{
  char* at = strchr(str, '@');
  if(at) {
    *at = 0;
    setenv("NULLMAILER_HOST", at+1, 1);
  }
  setenv("NULLMAILER_USER", str, 1);
  return true;
}

bool use_header = false;

// Flags:
// -Ffullname -- set the full name of the sender
// -fname -- set the envelope sender address
// -t -- read the header for recipients and ignore the command line arguments
int parseargs(int argc, char* argv[])
{
  int ch;
  while((ch = getopt(argc, argv, "tF:f:")) != -1) {
    switch(ch) {
    case 't':
      use_header = true;
      break;
    case 'F':
      setenv("NULLMAILER_NAME", optarg, 1);
      break;
    case 'f':
      if(!setenvelope(optarg))
	return -1;
      break;
    default:
      ferr.write("sendmail: Invalid option: ");
      ferr.writeln(argv[optind]);
      return -1;
    }
  }
  return optind;
}

int main(int argc, char* argv[])
{
  if(chdir(BIN_DIR) == -1) {
    ferr.write("sendmail: Could not change directory to ");
    ferr.writeln(BIN_DIR);
    return 1;
  }
  
  int i = parseargs(argc, argv);
  if(i == -1)
    return 1;

  char* newargv[argc-i+2];
  newargv[0] = "nullmailer-inject";
  int j = 1;
  if(use_header)
    newargv[j++] = "-b";
  else
    newargv[j++] = "-e";
  for(; i < argc; i++)
    newargv[j++] = argv[i];
  newargv[j] = 0;

  execv(newargv[0], newargv);
  ferr.writeln("sendmail: Could not exec nullmailer-inject.");
  return 1;
}
